<?php
/*
    ---------------------------------------------------------------------------   
     awsDB.php 
    ---------------------------------------------------------------------------
     Database connections and support functions for Microsoft SQL Server.
    ---------------------------------------------------------------------------
     Copyright 2013, AppGeo Solutions Ltd
     Godwin Badu-Marfo, started 31 January 2013
    --------------------------------------------------------------------------- 
*/

	// ----- library references
	include_once('config.inc.php');
	include_once('lookup.inc.php');
	include_once('awsCore.inc.php');
class awsDB
{
	
	// ----- object properties and variables ----------------------------------
  	static $handle = null;
	static $type = "";
	// ------------------------------------------------------------------------
  
  
	function _construct()
	/*
  	------------------------------------------------------------------------
	 1st class constructor
	 * returns connection to the database server
	------------------------------------------------------------------------
  	*/
	{
		$this->Connect();
		if (isset(config::$type))
		{
			self::$type = config::$type;
		}
	}
	

	static function Connect()
	/*
  	------------------------------------------------------------------------
	 Connect()
  	------------------------------------------------------------------------
	 sets connection to the database.
	 $connectionInfo = array("UID" => $uid, "PWD" => $pwd, "Database"=>"AdventureWorks");
	------------------------------------------------------------------------
    */
	{   
		try
		{
            switch (config::$type)
            {
                case 0:
                $serverName = config::$server;			

                $connectionInfo = array("Database"=>config::$db, "Uid"=>config::$username, "Pwd"=>config::$password);

                //self::$handle = sqlsrv_connect($serverName, $connectionInfo);

                break;

                case 1:
                self::$handle = pg_connect("host=".config::$server." port=".config::$port." dbname=".config::$db." 
                                            user=".config::$username." password=".config::$password);
                break;

                case 2:
                
                // set connection mysql server
                self::$handle =  mysqli_connect(config::$server, config::$username, config::$password, config::$db);
                break;
            }
		
			if( self::$handle === false )
			{
				awsCore::redMessage('Unable to connect to server. Please contact System Administrator!');
				self::DatabaseError();
			}
			else 
			{
				return self::$handle;
			}
		}
		catch ( Exception $e )
		{
		  return $e->getMessage();
		}
	}
	
	
	static function DatabaseError()
	/*
  	------------------------------------------------------------------------
	 Error()
  	------------------------------------------------------------------------
	 displays database error message
	------------------------------------------------------------------------
    */
	{
		switch (self::$type)
		{
			case 0:
   		 	die("<p>"."Database error: " . sqlsrv_errors()."</p>");
			break;
			
			case 1:
			die("<p>"."Database error: ". pg_last_error()."</p>");
			break;
            
            case 2:
            die("<p>"."Database error: ". mysqli_error()."</p>");
            break;
		}
		
	}
	
	static function CloseDB()
	/*
  	------------------------------------------------------------------------
	 CloseDB()
  	------------------------------------------------------------------------
	 closes connection to the database.
	------------------------------------------------------------------------
    */
	{
		if (!self::$handle) {exit;}			// ----- check if connection exists
		
		switch (self::$handle)				// ----- close connection to the database
		{
			case 0:
   		 	sqlsrv_close(self::$handle);
			break;
			
			case 1:
			pg_close(self::$handle);
			break;
            
            case 2:
            mysqli_close(self::$handle);
            break;
		}
		
	}
    
    
    //call_user_func_array(array($stmt, 'bind_param'), refValues($params));

    function refValues($arr){
        if (strnatcmp(phpversion(),'5.5') >= 0) //Reference is required for PHP 5.3+
        {
            $refs = array();
            foreach($arr as $key => $value)
                $refs[$key] = &$arr[$key];
            return $refs;
        }
        return $arr;
    }

    
	
	public static function executeQuery($query, $param = null)
	/*
  	---------------------------------------------------------------------------
   	execute()
  	---------------------------------------------------------------------------
   	Executes the sql command.
   	@param string $query script to execute and returns results.
	@return The executed recordset.
  	---------------------------------------------------------------------------   
  	*/ 
	{
        try
        {
		if (strlen($query) < 1){exit;}	// ----- checks query string is set.
		
		//if (!isset(self::$handle)){self::$handle = self::Connect();} // --- set connection to server
		
        $conn = new mysqli(config::$server, config::$username, config::$password, config::$db);
        
		switch (config::$type)
		{
			/*case 0:
			if ($param != null) 
            {
                $recordset = sqlsrv_prepare(self::$handle, $query, $param);
            }
			else
            {
                $recordset = sqlsrv_prepare(self::$handle, $query);
            }
			
			//if ($recordset = sqlsrv_query(self::$handle, $query)){return true;};	 	// ------ executes the query statement.	
			if (!$recordset)
            {
                awsCore::errorMessage('Problem Occurred in preparing Statement !'.self::DatabaseError()); 
                return false;
            }
			else
			{
				if ($stmt = sqlsrv_execute ($recordset))
                {
                    return true;
                }
				else
                {
                    awsCore::errorMessage('Problem Occurred in executing Statement !'.self::DatabaseError()); 
                    return false;
                }
			}
			 
			CloseDB();																// ------ close connection to the database.
			break;*/
            
            case 2:
            //prepare statements
            if ($param != null){
                $stmt = $conn->prepare($query);
            
                if ($stmt)
                {  
                    call_user_func_array(array($stmt, 'bind_param'), self::refValues($param));
                }

                //execute statement
                $res = $stmt->execute();
            }
            else{
                $stmt = $conn->prepare($query);
                $res = $stmt->execute();
            }
            
            if ($res)
            {
                return true;
            }
            else
            {
                return false;
            }
            break;
		}
        }
        catch (Exception $e)
        {
            echo $e->getMessage();	
        }
	}
	
	
	public static function fetchRecordset($query, $param = null)
	/*
  	---------------------------------------------------------------------------
   	execute()
  	---------------------------------------------------------------------------
   	Executes the sql command.
   	@param string $query script to execute and returns results.
	@return The executed recordset.
  	---------------------------------------------------------------------------   
  	*/ 
	{
		try
		{
		if (strlen($query) < 1) 					// ----- checks query string is set.
		{
			exit; 
		}	
		
		if (!isset(self::$handle))
		{
			self::$handle = self::Connect();
		}
		
		switch (config::$type)
		{
			case 0:
			/*if ($param != null) {$recordset = sqlsrv_query(self::$handle, $query, $param);}
			$recordset = sqlsrv_query(self::$handle, $query);					// ------ executes the query statement.	
			break;*/
            
            case 2:
            $recordset = mysqli_query(self::$handle, $query);
            
            if ($recordset)
            {
                return $recordset;   
            }
            else
            {
                awsCore::errorMessage('Problem Occurred in executing Statement !'.self::DatabaseError());
            }
            break;
           
            /*if (!$recordset) {awsCore::errorMessage('Problem Occurred in executing Statement !'.self::DatabaseError());}
            else {die ('recordset :'.$recordset[0]);}
			return $recordset; */
			//CloseDB();															// ------ close connection to the database.
		}
		}
		catch (Exception $e)
		{
			echo $e->getMessage();	
		}
	}
	
	
	public static function executeScalar($query)
	/*
  	---------------------------------------------------------------------------
   	executeScalar()
  	---------------------------------------------------------------------------
   	Executes and returns resultset as a scalar variable.
   	@param string $query script to execute and returns results.
	@return The array to be returned as numeric
  	---------------------------------------------------------------------------   
  	*/ 
	{
        $ret = '';
        
        try
        {
            $conn = self::Connect();
            
            switch (config::$type)
            {
                case 2:
                $recordset = mysqli_query($conn, $query);
                
                while ($row = mysqli_fetch_row($recordset)){
                    $ret = $row[0];
                }
                break;
            }
            
        }
        catch (Exception $e)
        {
            print $e->getMessage();   
        }
        
        return $ret;
	}
	
	
    public static function fetchRowCount($query)
    {
        try
        {
            $rs = self::fetchRecordset($query);
		      
            if (!$rs)
            {
                die('problem occurred in executing statement. Please contact system administrator!');
                exit;  
            }
            switch (config::$type)
            {
                case 2: 
                $rowcnt = mysqli_num_rows($rs);
                break;
            }

            return $rowcnt;
        }
        catch (Exception $e)
        {
            print $e->getMessage();   
        }
    }
    
    
	
	public static function fetchROWID($table, $field, $rowid, $value)
	{
		$query = "SELECT $rowid FROM $table WHERE $field = $value ";		
	}
	
	
	public static function fetchArrayNumeric($query, $param='')
	/*
  	---------------------------------------------------------------------------
   	fetchArray()
  	---------------------------------------------------------------------------
   	Executes and returns resultset as array
   	@param string $query script to execute and returns results.
	@return The array to be returned as associative
  	---------------------------------------------------------------------------   
  	*/ 
	{
		$stmt = self::fetchRecordset($query, $param);
		$rows[] = [];
        
        switch (self::$type)
		{
			case 0:
                /*while($row = sqlsrv_fetch_array($stmt, SQLSRV_FETCH_NUMERIC))
                {
                    $rows[] = $row;
                }*/
            break;
            case 2: 
            while($row = mysqli_fetch_array($stmt, MYSQL_NUM))
            {
                $rows[] += $row;
            }
            break;
        }

		return $rows;
	}
	
	public static function fetchArrayAssoc($query, $param=null)
	/*
  	---------------------------------------------------------------------------
   	fetchArray()
  	---------------------------------------------------------------------------
   	Executes and returns resultset as array
   	@param string $query script to execute and returns results.
	@return The array to be returned as associative
  	---------------------------------------------------------------------------   
  	*/ 
	{
		$stmt = self::fetchRecordset($query, $param);
		$rows[] = [];
            
        switch (self::$type)
		{
			case 0:
                /*while($row = sqlsrv_fetch_array($stmt, SQLSRV_FETCH_ASSOC))
                {
                    $rows[] = $row;
                }*/
            break;
            case 2: 
                while($row = mysqli_fetch_array($stmt, MYSQL_ASSOC))
                {
                    $rows[] += $row;
                }
            break;
        }

		return $rows;
	}
	
	
	public static function sanitizeName($txt) 
	/*
	  ------------------------------------------------------------------------
	   sanitizeName()
	  ------------------------------------------------------------------------
	   Check string for correct coding and filter out forbidden characters
	   @param string $txt Text string to be checked.
	   @return Returns sanitized string.
	  ------------------------------------------------------------------------
	*/
	{
		if (!mb_check_encoding($txt, 'UTF-8')) 
		{
			die('Character coding   with  errors at input value.');
		}
		
		mb_regex_encoding('UTF-8');
		return mb_ereg_replace("[^-[:alnum:]',  ]", '', $txt);
	}

	
	
	public static function cleanInput($str)
	/*
  	---------------------------------------------------------------------------
   	cleanInput()
  	---------------------------------------------------------------------------
   	Clean an input string getting it ready to be used in a SQL statement..
   	@param string $str The input string to clean.
	@return The cleaned up string.
  	---------------------------------------------------------------------------   
  	*/ 
	{
		switch(self::$type)
		{
			case 0:
			return self::sanitizeName($str);
			break;
			
			case 1:
			return trim(pg_escape_string(utf8_encode($str)));
			break;
			
			case 2:
			return trim(mysqli_real_escape_string(utf8_encode($str)));
			break;	
		}
	}
	
	public static function cleanDataRead($str)
	/*
  	---------------------------------------------------------------------------
   	cleanDataRead()
  	---------------------------------------------------------------------------
   	Clean data read from the database, basically apply utf8_decode()
   	@param string $str Raw string read from the database.
	@return The string cleaned version of the data.
  	---------------------------------------------------------------------------   
  	*/
	{
		switch (self::$type)
		{
			case 0:
			return htmlspecialchars($str);
			break;
			
			case 1:
			return utf8_decode($str);
			break;
		}
	}
	
	
	public static function executeInsert($tableName, $data)
	/*
  	---------------------------------------------------------------------------
   	generateInsert()
  	---------------------------------------------------------------------------
   	Generates the insert statement script.
   	@param string $tableName Table Name of table to insert into.
	@param array $array Data array to be inserted in Key, value pair.
	@return Returns bit on success.
	($data = array('field1'=> value1, 'field2'=> 'value2')
  	---------------------------------------------------------------------------   
  	*/
	{
		    $columns = "";
            $values = "";
            $returnCode = true;
			
			if (!is_array($data)) {awsCore::errorMessage('Please specify a valid array data.');}
			
			foreach ($data as $key => $value)
			{
				$columns += printf('%s', $key);
				
				// ----- check for value data type to print.
				if (gettype($value)=== integer || gettype($value) === double)
				{
					$values += printf('%d,', $value);
				}
				else
				{
					$values += printf('%s,', $value);	
				}
			}
			$columns = substr($columns, 0 , -1);
			$values = substr($values, 0, -1);
			
			$stmt = printf('insert into $s ($s) values ($s)', $tableName, $columns, $values);		
	}
	
	
	public static function executeJSON($lookup)
	/*
  	---------------------------------------------------------------------------
   	executeJSON()
  	---------------------------------------------------------------------------
   	execute lookup to return JSON data.
  	---------------------------------------------------------------------------   
  	*/
	{
		if (strlen($lookup) > 0)
		{
			//$result = self::executeQuery($lookup);	
			$result = self::fetchArrayAssoc($lookup);
			echo json_encode($result);
		}
	}
	
	public static function writeJsonToFile($file = 'd')
	/*
  	---------------------------------------------------------------------------
   	writeJsonToFile()
  	---------------------------------------------------------------------------
   	Writes JSON to file
  	---------------------------------------------------------------------------   
  	*/
	{
		if ($file == 'd')
		{
			$jsonurl = ROOT_DIR.'library/data/data.json';	
		}
		else
		{
			$jsonurl = $file;
		}
		
		$json = file_get_contents($jsonurl,0,null,null);
		$json_output = json_decode($json);
	}
}
?>